-module(game_logic).
-compile(export_all).
-export([validate_play/2, calculate_card_value/1, evaluate_hand_strength/1,
         find_all_possible_patterns/1, analyze_card_pattern/1]).

-define(CARD_VALUES, #{
    "3" => 3, "4" => 4, "5" => 5, "6" => 6, "7" => 7, "8" => 8, "9" => 9,
    "10" => 10, "J" => 11, "Q" => 12, "K" => 13, "A" => 14, "2" => 15,
    "小王" => 16, "大王" => 17
}).

%% 牌型定义
-record(card_pattern, {
    type,        % single | pair | triple | triple_with_one | straight | 
                 % straight_pair | airplane | airplane_with_wings | four_with_two | bomb | rocket
    value,       % 主牌值
    length = 1,  % 顺子长度
    extra = []   % 附加牌
}).

%% 验证出牌是否合法
validate_play(Cards, LastPlay) ->
    case analyze_card_pattern(Cards) of
        {invalid, _} -> 
            false;
        {Pattern, Value} -> 
            case analyze_card_pattern(LastPlay) of
                {Pattern, LastValue} when Value > LastValue -> 
                    true;
                {_, _} -> 
                    check_special_cases(Pattern, Value, LastPlay);
                _ -> 
                    false
            end
    end.

%% 分析牌型
analyze_card_pattern(Cards) ->
    Grouped = group_cards(Cards),
    case identify_pattern(Grouped) of
        {Type, Value} when Type =/= invalid ->
            {#card_pattern{type = Type, value = Value}, Value};
        _ ->
            {invalid, 0}
    end.

%% 计算手牌价值
calculate_card_value(Cards) ->
    {Pattern, BaseValue} = analyze_card_pattern(Cards),
    case Pattern#card_pattern.type of
        bomb -> BaseValue * 2;
        rocket -> 1000;
        _ -> BaseValue
    end.

%% 评估手牌强度
evaluate_hand_strength(Cards) ->
    Patterns = find_all_possible_patterns(Cards),
    BaseScore = lists:foldl(
        fun(Pattern, Acc) ->
            Acc + calculate_pattern_value(Pattern)
        end,
        0,
        Patterns
    ),
    adjust_score_by_combination(BaseScore, Cards).

%% 找出所有可能的牌型组合
find_all_possible_patterns(Cards) ->
    Grouped = group_cards(Cards),
    Singles = find_singles(Grouped),
    Pairs = find_pairs(Grouped),
    Triples = find_triples(Grouped),
    Straights = find_straights(Cards),
    StraightPairs = find_straight_pairs(Grouped),
    Airplanes = find_airplanes(Grouped),
    Bombs = find_bombs(Grouped),
    Rockets = find_rockets(Cards),
    Singles ++ Pairs ++ Triples ++ Straights ++ StraightPairs ++ 
    Airplanes ++ Bombs ++ Rockets.

%% 查找单牌
find_singles(Grouped) ->
    [{#card_pattern{type = single, value = maps:get(Number, ?CARD_VALUES)}, Number} || 
        {Number, 1} <- maps:to_list(Grouped)].

%% 查找对子
find_pairs(Grouped) ->
    [{#card_pattern{type = pair, value = maps:get(Number, ?CARD_VALUES)}, Number} || 
        {Number, 2} <- maps:to_list(Grouped)].

%% 查找三张
find_triples(Grouped) ->
    [{#card_pattern{type = triple, value = maps:get(Number, ?CARD_VALUES)}, Number} || 
        {Number, 3} <- maps:to_list(Grouped)].

%% 查找顺子
find_straights(Cards) ->
    Values = [V || {_, V} <- Cards],
    CardValues = [maps:get(V, ?CARD_VALUES) || V <- Values],
    SortedValues = lists:sort(CardValues),
    % 移除重复值
    UniqueValues = lists:usort(SortedValues),
    % 查找所有可能的顺子
    find_all_straights(UniqueValues, 5).

%% 查找所有可能的顺子组合
find_all_straights(Values, MinLength) when length(Values) >= MinLength ->
    MaxValue = lists:max(Values),
    % 确保没有2和王
    case MaxValue < maps:get("2", ?CARD_VALUES) of
        true ->
            % 找出所有可能的起始点
            PossibleStarts = find_straight_starts(Values, MinLength),
            % 从每个起始点生成顺子
            lists:flatten([generate_straights(Values, Start, MinLength) || Start <- PossibleStarts]);
        false ->
            % 如果有2或王，需要过滤掉它们再查找
            FilteredValues = [V || V <- Values, V < maps:get("2", ?CARD_VALUES)],
            find_all_straights(FilteredValues, MinLength)
    end;
find_all_straights(_, _) ->
    [].

%% 查找所有可能的顺子起始点
find_straight_starts(Values, MinLength) ->
    % 检查每个值是否可以作为顺子的起始点
    [V || V <- Values, can_start_straight(V, Values, MinLength)].

%% 检查是否可以从某个值开始形成顺子
can_start_straight(Start, Values, MinLength) ->
    RequiredValues = lists:seq(Start, Start + MinLength - 1),
    lists:all(fun(V) -> lists:member(V, Values) end, RequiredValues).

%% 从起始点生成顺子
generate_straights(Values, Start, MinLength) ->
    % 找出从Start开始的最长连续序列
    MaxLength = find_max_straight_length(Start, Values),
    % 生成所有可能长度的顺子
    [generate_straight_pattern(Start, Len) || Len <- lists:seq(MinLength, MaxLength)].

%% 查找最长顺子长度
find_max_straight_length(Start, Values) ->
    find_max_straight_length(Start, Values, 0).

find_max_straight_length(Current, Values, Length) ->
    case lists:member(Current, Values) of
        true -> find_max_straight_length(Current + 1, Values, Length + 1);
        false -> Length
    end.

%% 生成顺子牌型
generate_straight_pattern(Start, Length) ->
    Value = Start + Length - 1, % 顺子的最大值
    {#card_pattern{type = straight, value = Value, length = Length}, straight}.

%% 查找连对
find_straight_pairs(Grouped) ->
    % 找出所有对子
    Pairs = [{N, N} || {N, C} <- maps:to_list(Grouped), C =:= 2],
    % 按值排序
    SortedPairs = lists:sort(fun({_, V1}, {_, V2}) -> V1 =< V2 end, Pairs),
    % 查找连续的对子
    find_consecutive_pairs(SortedPairs, 3).

%% 查找连续的对子
find_consecutive_pairs(Pairs, MinLength) ->
    case length(Pairs) >= MinLength of
        true ->
            Values = [maps:get(N, ?CARD_VALUES) || {N, _} <- Pairs],
            % 确保没有2和王
            case lists:max(Values) < maps:get("2", ?CARD_VALUES) of
                true ->
                    find_all_consecutive_pairs(Pairs, MinLength);
                false ->
                    % 过滤掉2和王
                    FilteredPairs = [{N, V} || {N, V} <- Pairs, maps:get(N, ?CARD_VALUES) < maps:get("2", ?CARD_VALUES)],
                    find_consecutive_pairs(FilteredPairs, MinLength)
            end;
        false ->
            []
    end.

%% 查找所有连续对子组合
find_all_consecutive_pairs(Pairs, MinLength) ->
    PairValues = [maps:get(N, ?CARD_VALUES) || {N, _} <- Pairs],
    % 找出所有可能的起始点
    PossibleStarts = find_straight_starts(PairValues, MinLength),
    % 从每个起始点生成连对
    lists:flatten([generate_straight_pair_pattern(Pairs, Start, MinLength) || Start <- PossibleStarts]).

%% 生成连对牌型
generate_straight_pair_pattern(Pairs, Start, MinLength) ->
    % 找出从Start开始的最长连续序列
    MaxLength = find_max_straight_length(Start, [maps:get(N, ?CARD_VALUES) || {N, _} <- Pairs]),
    % 生成所有可能长度的连对
    [generate_straight_pair(Pairs, Start, Len) || Len <- lists:seq(MinLength, MaxLength)].

%% 生成连对
generate_straight_pair(Pairs, Start, Length) ->
    EndValue = Start + Length - 1,
    {#card_pattern{type = straight_pair, value = EndValue, length = Length}, straight_pair}.

%% 查找飞机
find_airplanes(Grouped) ->
    % 查找所有三张
    Triples = [{N, maps:get(N, ?CARD_VALUES)} || {N, C} <- maps:to_list(Grouped), C =:= 3],
    % 按值排序
    SortedTriples = lists:sort(fun({_, V1}, {_, V2}) -> V1 =< V2 end, Triples),
    % 查找连续的三张
    find_consecutive_triples(SortedTriples, Grouped, 2).

%% 查找连续的三张
find_consecutive_triples(Triples, Grouped, MinLength) ->
    case length(Triples) >= MinLength of
        true ->
            Values = [V || {_, V} <- Triples],
            % 确保没有2和王
            case lists:max(Values) < maps:get("2", ?CARD_VALUES) of
                true ->
                    find_all_consecutive_triples(Triples, Grouped, MinLength);
                false ->
                    % 过滤掉2和王
                    FilteredTriples = [{N, V} || {N, V} <- Triples, V < maps:get("2", ?CARD_VALUES)],
                    find_consecutive_triples(FilteredTriples, Grouped, MinLength)
            end;
        false ->
            []
    end.

%% 查找所有连续三张组合
find_all_consecutive_triples(Triples, Grouped, MinLength) ->
    TripleValues = [V || {_, V} <- Triples],
    % 找出所有可能的起始点
    PossibleStarts = find_straight_starts(TripleValues, MinLength),
    % 从每个起始点生成飞机
    lists:flatten([generate_airplane_pattern(Triples, Grouped, Start, MinLength) || Start <- PossibleStarts]).

%% 生成飞机牌型
generate_airplane_pattern(Triples, Grouped, Start, MinLength) ->
    % 找出从Start开始的最长连续序列
    MaxLength = find_max_straight_length(Start, [V || {_, V} <- Triples]),
    % 生成所有可能长度的飞机
    [generate_airplane(Triples, Grouped, Start, Len) || Len <- lists:seq(MinLength, MaxLength)].

%% 生成飞机
generate_airplane(Triples, Grouped, Start, Length) ->
    EndValue = Start + Length - 1,
    % 检查是否有翅膀
    OtherCards = maps:to_list(maps:without([N || {N, _} <- Triples], Grouped)),
    HasWings = length(OtherCards) > 0,
    PlaneType = case HasWings of
        true -> plane_one;
        false -> plane
    end,
    {#card_pattern{type = PlaneType, value = EndValue, length = Length}, PlaneType}.

%% 查找炸弹
find_bombs(Grouped) ->
    [{#card_pattern{type = bomb, value = maps:get(Number, ?CARD_VALUES)}, Number} || 
        {Number, 4} <- maps:to_list(Grouped)].

%% 查找火箭
find_rockets(Cards) ->
    % 检查是否有大小王
    case lists:member({joker, "小王"}, Cards) andalso lists:member({joker, "大王"}, Cards) of
        true -> [{#card_pattern{type = rocket, value = 1000}, rocket}];
        false -> []
    end.

%% 内部辅助函数

group_cards(Cards) ->
    lists:foldl(
        fun({_, Number}, Acc) ->
            maps:update_with(
                Number,
                fun(Count) -> Count + 1 end,
                1,
                Acc
            )
        end,
        #{},
        Cards
    ).

identify_pattern(Grouped) ->
    case maps:size(Grouped) of
        1 -> 
            [{Number, Count}] = maps:to_list(Grouped),
            case Count of
                1 -> {single, maps:get(Number, ?CARD_VALUES)};
                2 -> {pair, maps:get(Number, ?CARD_VALUES)};
                3 -> {triple, maps:get(Number, ?CARD_VALUES)};
                4 -> {bomb, maps:get(Number, ?CARD_VALUES)};
                _ -> {invalid, 0}
            end;
        _ ->
            identify_complex_pattern(Grouped)
    end.

identify_complex_pattern(Grouped) ->
    case find_straight_pattern(Grouped) of
        {ok, Pattern} -> Pattern;
        false ->
            case find_airplane_pattern(Grouped) of
                {ok, Pattern} -> Pattern;
                false ->
                    case find_four_with_two(Grouped) of
                        {ok, Pattern} -> Pattern;
                        false -> {invalid, 0}
                    end
            end
    end.

%% 查找飞机牌型
find_airplane_pattern(Grouped) ->
    % 查找所有三张
    Triples = [{N, C} || {N, C} <- maps:to_list(Grouped), C =:= 3],
    case length(Triples) >= 2 of
        true ->
            % 检查是否连续
            Numbers = [N || {N, _} <- Triples],
            SortedNumbers = lists:sort(Numbers),
            case is_consecutive_values(SortedNumbers) of
                true ->
                    MaxValue = lists:max([maps:get(N, ?CARD_VALUES) || N <- Numbers]),
                    {ok, {airplane, MaxValue}};
                false ->
                    false
            end;
        false ->
            false
    end.

%% 查找四带二牌型
find_four_with_two(Grouped) ->
    % 查找四张
    Fours = [{N, C} || {N, C} <- maps:to_list(Grouped), C =:= 4],
    case Fours of
        [{Number, 4}] ->
            % 检查是否有两张单牌或一对
            OtherCards = maps:to_list(maps:remove(Number, Grouped)),
            case length(OtherCards) of
                1 -> 
                    [{_, 2}] = OtherCards,
                    {ok, {four_with_two, maps:get(Number, ?CARD_VALUES)}};
                2 -> 
                    [{_, 1}, {_, 1}] = OtherCards,
                    {ok, {four_with_two, maps:get(Number, ?CARD_VALUES)}};
                _ -> 
                    false
            end;
        _ ->
            false
    end.

%% 检查值是否连续
is_consecutive_values(Values) ->
    case Values of
        [] -> true;
        [_] -> true;
        _ ->
            SortedValues = lists:sort(Values),
            lists:all(
                fun({A, B}) -> B - A =:= 1 end,
                lists:zip(SortedValues, lists:sublist(SortedValues, 2, length(SortedValues) - 1))
            )
    end.

find_straight_pattern(Grouped) ->
    Numbers = lists:sort(maps:keys(Grouped)),
    case is_consecutive(Numbers) of
        true when length(Numbers) >= 5 ->
            MaxValue = lists:max([maps:get(N, ?CARD_VALUES) || N <- Numbers]),
            {ok, {straight, MaxValue}};
        _ ->
            false
    end.

is_consecutive(Numbers) ->
    case Numbers of
        [] -> true;
        [_] -> true;
        [First|Rest] ->
            lists:all(
                fun({A, B}) -> maps:get(B, ?CARD_VALUES) - maps:get(A, ?CARD_VALUES) =:= 1 end,
                lists:zip(Numbers, Rest)
            )
    end.

calculate_pattern_value(Pattern) ->
    case Pattern of
        #card_pattern{type = single, value = V} -> V;
        #card_pattern{type = pair, value = V} -> V * 2;
        #card_pattern{type = triple, value = V} -> V * 3;
        #card_pattern{type = bomb, value = V} -> V * 4;
        #card_pattern{type = rocket} -> 1000;
        #card_pattern{type = straight, value = V, length = L} -> V * L;
        _ -> 0
    end.

adjust_score_by_combination(BaseScore, Cards) ->
    CombinationBonus = case length(Cards) of
        N when N =< 5 -> BaseScore * 1.2;
        N when N =< 10 -> BaseScore * 1.1;
        _ -> BaseScore
    end,
    round(CombinationBonus).

check_special_cases(Pattern, Value, LastPlay) ->
    {LastPattern, _} = analyze_card_pattern(LastPlay),
    case Pattern#card_pattern.type of
        bomb when LastPattern#card_pattern.type =/= bomb andalso LastPattern#card_pattern.type =/= rocket -> 
            true;
        bomb when LastPattern#card_pattern.type =:= bomb andalso Value > LastPattern#card_pattern.value -> 
            true;
        rocket -> 
            true;
        _ -> 
            false
    end.